import mariadb
import json
from pathlib import Path
from time import perf_counter
from pprint import pprint
from itertools import chain, repeat

conn = mariadb.connect(
       user="root",
       password="root",
       host="localhost",
       port=3306,
       database='imdb'
    )

c = conn.cursor()

# c.execute('TRUNCATE TABLE title') # (only for repeated cell running in notebook)
# c.execute('TRUNCATE TABLE in_genre') # (only for repeated cell running in notebook)
c.execute('SELECT id FROM title')
title_PKs = set(chain.from_iterable(c.fetchall()))

with open('enums.json', encoding='utf-8') as enums_f:
    enums = json.load(enums_f)
for key in enums:  # transform to lookup table for ids: enums['genre']['Comedy'] => genre.id
    enums[key] = {name: i for i, name in enumerate(enums[key], 1)}


def idstrip(idstr, prefix_len=2):
    return int(idstr.strip()[prefix_len:])

def id_helper(parts, column, col_name, prefix_len):
    return idstrip(parts[column[col_name]], prefix_len)

def value_helper(parts, column, col_name, cls):
    cv = parts[column[col_name]].strip()
    return cls(cv) if cv != r'\N' else None

def enum_helper(parts, column, col_name, enum_name):
    v = value_helper(parts, column, col_name, str)
    return enums[enum_name][v] if v is not None else None

# valhelp and idhelp uses the parts and column directly from current scope
valhelp = lambda col_name, cls=str: value_helper(parts, column, col_name, cls)
idhelp = lambda  col_name, prefix_len=2: id_helper(parts, column, col_name, prefix_len)
enmhelp = lambda col_name, enum_name: enum_helper(parts, column, col_name, enum_name)


# c.execute('TRUNCATE TABLE title') # (only for repeated cell running in notebook)
# c.execute('TRUNCATE TABLE in_genre') # (only for repeated cell running in notebook)

c.execute('SELECT id FROM title')
title_PKs = set(chain.from_iterable(c.fetchall()))

t1 = perf_counter()
with open(Path('data') / 'title.basics.tsv', encoding='utf-8') as f:
    column = {}
    for i, p in enumerate(f.readline().strip().split('\t')):
        column[p.strip()] = i
    for li, line in enumerate(f):
        parts = line.strip().split('\t')
        id_ = idhelp('tconst')  # ommit prefix
        if id_ not in title_PKs:
            title_type_id = enmhelp('titleType', 'title_type')
            primary_title = valhelp('primaryTitle')
            original_title = valhelp('originalTitle')
            is_adult = valhelp('isAdult', int)
            start_year = valhelp('startYear', int)
            end_year = valhelp('endYear', int)
            runtime_minutes = valhelp('runtimeMinutes', int)
            c.execute('INSERT INTO title '
                      '(id, title_type_id, primary_title, original_title, is_adult, start_year, end_year, runtime_minutes) '
                      'VALUES (?,?,?,?,?,?,?,?)',
                      (id_, title_type_id, primary_title, original_title, is_adult, start_year, end_year, runtime_minutes))
            # print('insert id:', id_)

        glist = parts[column['genres']].strip().split(',')
        if len(glist) and glist[0] != '\\N':
            found_all = False
            if id_ in title_PKs:
                c.execute('SELECT COUNT(title_id) FROM in_genre WHERE title_id=?', (id_,))
                found_all = c.fetchone()[0] == len(glist)
            if not found_all:
                # print('not found all for:', id_)
                values = tuple(chain.from_iterable((id_, enums['genre'][p]) for p in glist if len(p)))
                if len(values):
                    qmarks = ','.join(repeat('(?,?)', len(values) // 2))
                    c.execute(f'INSERT IGNORE INTO in_genre (title_id, genre_id) VALUES {qmarks}', values)

print(perf_counter() - t1, 's')
